<?php

if (!defined('BASEPATH'))
    exit('No direct script access allowed');
/**
 * CodeIgniter
 *
 * An open source application development framework for PHP 5.1.6 or newer
 *
 * @package                CodeIgniter
 * @author                ExpressionEngine Dev Team
 * @copyright        Copyright (c) 2008 - 2011, EllisLab, Inc.
 * @license                http://codeigniter.com/user_guide/license.html
 * @link                http://codeigniter.com
 * @since                Version 1.0
 * @filesource
 */
// ------------------------------------------------------------------------

/**
 * Database Cache Class
 *
 * @category        Database
 * @author                ExpressionEngine Dev Team
 * @link                http://codeigniter.com/user_guide/database/
 */
class CI_DB_Cache {

    var $CI;
    var $db;        // allows passing of db object so that multiple database connections and returned db objects can be supported
    var $prefix;

    /**
     * Constructor
     *
     * Grabs the CI super object instance so we can access it.
     *
     */
    function __construct(&$db) {
        // Assign the main CI object to $this->CI
        // and load the file helper since we use it a lot
        $this->CI = & get_instance();
        $this->db = & $db;
        $this->CI->load->helper('file');
    }

    // --------------------------------------------------------------------

    /**
     * Set Cache Directory Path
     *
     * @access        public
     * @param        string        the path to the cache directory
     * @return        bool
     */
    function check_path($path = '') {
        if ($path == '') {
            if ($this->db->cachedir == '') {
                return $this->db->cache_off();
            }

            $path = $this->db->cachedir;
        }
        if (!is_dir($path)) {
            @mkdir($path, DIR_WRITE_MODE);
        }
        
        // Add a trailing slash to the path if needed
        $path = preg_replace("/(.+?)\/*$/", "\\1/", $path);

        if (!is_dir($path) OR !is_really_writable($path)) {
            // If the path is wrong we'll turn off caching
            return $this->db->cache_off();
        }

        $this->db->cachedir = $path;
        return TRUE;
    }

    // --------------------------------------------------------------------

    /**
     * Retrieve a cached query
     *
     * The URI being requested will become the name of the cache sub-folder.
     * An MD5 hash of the SQL statement will become the cache file name
     *
     * @access        public
     * @return        string
     */
    function read($sql) {
        if (!$this->check_path()) {
            return $this->db->cache_off();
        }

        if ($this->prefix) {
            $filepath = $this->db->cachedir . $this->prefix . '/' . md5(trim($sql));
        } else {
            $filepath = $this->db->cachedir . md5(trim($sql));
        }

        if (FALSE === ($cachedata = read_file($filepath))) {
            return FALSE;
        }

        return $this->_unserialize($cachedata);
    }

    // --------------------------------------------------------------------

    /**
     * Write a query to a cache file
     *
     * @access        public
     * @return        bool
     */
    function write($sql, $object) {
        if (!$this->check_path()) {
            return $this->db->cache_off();
        }

        if ($this->prefix) {
            $dir_path = $this->db->cachedir . $this->prefix . '/';
        } else {
            $dir_path = $this->db->cachedir;
        }

        $filename = md5(trim($sql));

        if (!@is_dir($dir_path)) {
            if (!@mkdir($dir_path, DIR_WRITE_MODE)) {
                return FALSE;
            } else {
                @copy(UPLOADSPATH . 'index.html', $dir_path . 'index.html');
            }

            @chmod($dir_path, DIR_WRITE_MODE);
        }

        if (write_file($dir_path . $filename, $this->_serialize($object)) === FALSE) {
            return FALSE;
        }

        @chmod($dir_path . $filename, FILE_WRITE_MODE);
        return TRUE;
    }

    // --------------------------------------------------------------------

    /**
     * Delete cache files within a query sql
     *
     * @access        public
     * @return        bool
     */
    function delete($sql = '') {
        if (!$this->check_path()) {
            return $this->db->cache_off();
        }

        if ($this->prefix) {
            $filepath = $this->db->cachedir . $this->prefix . '/' . md5(trim($sql));
        } else {
            $filepath = $this->db->cachedir . md5(trim($sql));
        }

        if (is_file($filepath))
            @unlink($filepath);
    }

    // --------------------------------------------------------------------

    /**
     * Delete all existing cache files
     *
     * @access        public
     * @return        bool
     */
    function delete_all() {
        if (!$this->check_path()) {
            return $this->db->cache_off();
        }
        delete_files($this->db->cachedir, TRUE);
    }

    // --------------------------------------------------------------------

    /**
     * Serialize an array
     *
     * This function first converts any slashes found in the array to a temporary
     * marker, so when it gets unserialized the slashes will be preserved
     *
     * @access        private
     * @param        array
     * @return        string
     */
    function _serialize($data) {
        if (is_array($data)) {
            foreach ($data as $key => $val) {
                if (is_string($val)) {
                    $data[$key] = str_replace('\\', '{{slash}}', $val);
                }
            }
        } else {
            if (is_string($data)) {
                $data = str_replace('\\', '{{slash}}', $data);
            }
        }

        return serialize($data);
    }

    // --------------------------------------------------------------------

    /**
     * Unserialize
     *
     * This function unserializes a data string, then converts any
     * temporary slash markers back to actual slashes
     *
     * @access        private
     * @param        array
     * @return        string
     */
    function _unserialize($data) {
        $data = @unserialize(strip_slashes($data));

        if (is_array($data)) {
            foreach ($data as $key => $val) {
                if (is_string($val)) {
                    $data[$key] = str_replace('{{slash}}', '\\', $val);
                }
            }

            return $data;
        }

        return (is_string($data)) ? str_replace('{{slash}}', '\\', $data) : $data;
    }

    /* New Prefix for Caches */

    function set_prefix($prefix = '') {
        $this->prefix = $prefix;
    }

    function delete_prefix() {
        delete_files($this->db->cachedir . $this->prefix . '/', TRUE);
    }

}

/* End of file DB_cache.php */
/* Location: ./system/database/DB_cache.php */